package dev.kske.chess;

import dev.kske.chess.piece.Bishop;
import dev.kske.chess.piece.King;
import dev.kske.chess.piece.Knight;
import dev.kske.chess.piece.Pawn;
import dev.kske.chess.piece.Piece;
import dev.kske.chess.piece.Piece.Color;
import dev.kske.chess.piece.Queen;
import dev.kske.chess.piece.Rook;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>Board.java</strong><br>
 * Created: <strong>01.07.2019</strong><br>
 * Author: <strong>Kai S. K. Engelbart</strong>
 */
public class Board {

	private Piece[][] boardArr;

	public Board() {
		boardArr = new Piece[8][8];
		initializeDefaultPositions();
	}

	public boolean move(int xPos, int yPos, int xDest, int yDest) {
		return move(new Move(xPos, yPos, xDest, yDest));
	}

	public boolean move(Move move) {
		Piece piece = getPos(move);
		if (piece == null || !piece.isValidMove(move)) return false;
		else {
			// Move piece
			setDest(move, getPos(move));
			setPos(move, null);
			return true;
		}
	}

	public Piece getPos(Move move) {
		return boardArr[move.xPos][move.yPos];
	}

	public Piece getDest(Move move) {
		return boardArr[move.xDest][move.yDest];
	}

	public void setPos(Move move, Piece piece) {
		boardArr[move.xPos][move.yPos] = piece;
	}

	public void setDest(Move move, Piece piece) {
		boardArr[move.xDest][move.yDest] = piece;
	}

	/**
	 * Initialized the board array with the default chess pieces and positions.
	 */
	private void initializeDefaultPositions() {
		// Initialize pawns
		for (int i = 0; i < 8; i++) {
			boardArr[i][1]	= new Pawn(Color.BLACK, this);
			boardArr[i][6]	= new Pawn(Color.WHITE, this);
		}

		// Initialize kings
		boardArr[4][0]	= new King(Color.BLACK, this);
		boardArr[4][7]	= new King(Color.WHITE, this);

		// Initialize queens
		boardArr[3][0]	= new Queen(Color.BLACK, this);
		boardArr[3][7]	= new Queen(Color.WHITE, this);

		// Initialize rooks
		boardArr[0][0]	= new Rook(Color.BLACK, this);
		boardArr[0][7]	= new Rook(Color.WHITE, this);
		boardArr[7][0]	= new Rook(Color.BLACK, this);
		boardArr[7][7]	= new Rook(Color.WHITE, this);

		// Initialize knights
		boardArr[1][0]	= new Knight(Color.BLACK, this);
		boardArr[1][7]	= new Knight(Color.WHITE, this);
		boardArr[6][0]	= new Knight(Color.BLACK, this);
		boardArr[6][7]	= new Knight(Color.WHITE, this);

		// Initialize bishops
		boardArr[2][0]	= new Bishop(Color.BLACK, this);
		boardArr[2][7]	= new Bishop(Color.WHITE, this);
		boardArr[5][0]	= new Bishop(Color.BLACK, this);
		boardArr[5][7]	= new Bishop(Color.WHITE, this);
	}

	/**
	 * @return The board array
	 */
	public Piece[][] getBoardArr() { return boardArr; }
}
