package dev.kske.chess.piece;

import dev.kske.chess.Board;
import dev.kske.chess.Move;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>Queen.java</strong><br>
 * Created: <strong>01.07.2019</strong><br>
 * Author: <strong>Kai S. K. Engelbart</strong>
 */
public class Queen extends Piece {

	public Queen(Color color, Board board) {
		super(color, board);
	}

	@Override
	public boolean isValidMove(Move move) {
		return ((move.isHorizontal() || move.isVertical()) || move.isDiagonal()) && isFreePath(move);
	}

	@Override
	protected boolean isFreePath(Move move) {
		if (move.isHorizontal()) {
			for (int i = move.xPos + move.xSign; i != move.xDest; i += move.xSign)
				if (board.getBoardArr()[i][move.yPos] != null) return false;
		} else if (move.isVertical()) {
			for (int i = move.yPos + move.ySign; i != move.yDest; i += move.ySign)
				if (board.getBoardArr()[move.xPos][i] != null) return false;
		} else {
			for (int i = move.xPos + move.xSign, j = move.yPos
					+ move.ySign; i != move.xDest; i += move.xSign, j += move.ySign)
				if (board.getBoardArr()[i][j] != null) return false;
		}
		return checkDestination(move);
	}

	@Override
	public Type getType() { return Type.QUEEN; }
}
